@@ -1,7 +1,7 @@
 # **************************************************************************
-# * Authors:     J.M. De la Rosa Trevin (jmdelarosa@cnb.csic.es)
+# * Authors:     Mohamad Harastani (mohamad.harastani@igbmc.fr)
+# *              J.M. De la Rosa Trevin (jmdelarosa@cnb.csic.es)
 # *              Slavica Jonic  (slavica.jonic@upmc.fr)
-# *              Mohamad Harastani (mohamad.harastani@upmc.fr)
 # *
 # * This program is free software; you can redistribute it and/or modify
 # * it under the terms of the GNU General Public License as published by
@@ -22,35 +22,24 @@
 # *  e-mail address 'scipion@cnb.csic.es'
 # *
 # **************************************************************************
+
 from continuousflex.protocols.data import PathData
-
-"""
-This module implement the wrappers around Xmipp CL2D protocol
-visualization program.
-"""
-
-from os.path import basename, join, exists
+from os.path import basename, join, exists, isfile
 import numpy as np
-
-from pwem.convert.atom_struct import cifToPdb
-from pyworkflow.utils import replaceBaseExt
-
-from pyworkflow.utils.path import cleanPath, makePath, cleanPattern
+from joblib import load
+from pyworkflow.utils.path import cleanPath, makePath
 from pyworkflow.viewer import (ProtocolViewer, DESKTOP_TKINTER, WEB_DJANGO)
 from pyworkflow.protocol.params import StringParam, LabelParam
 from pwem.objects import SetOfParticles
 from pwem.viewers import VmdView
 from pyworkflow.gui.browser import FileBrowserWindow
-
 from continuousflex.protocols.protocol_nma_dimred import FlexProtDimredNMA
-
 from continuousflex.protocols.data import Point, Data
-
 from continuousflex.viewers.nma_plotter import FlexNmaPlotter
-
 from continuousflex.viewers.nma_gui import ClusteringWindow, TrajectoriesWindow
 from pwem.utils import runProgram
 from pyworkflow.protocol import params
+from continuousflex.protocols import FlexProtDeepHEMNMAInfer
 
 FIGURE_LIMIT_NONE = 0
 FIGURE_LIMITS = 1
@@ -66,7 +55,7 @@ POINT_LIMITS_NONE = 0
 POINT_LIMITS = 1
 
 class FlexDimredNMAViewer(ProtocolViewer):
-    """ Visualization of results from the NMA protocol
+    """ Visualization of results from the NMA alingment dimred protocol (HEMNMA)
     """
     _label = 'viewer nma dimred'
     _targets = [FlexProtDimredNMA]
@@ -178,6 +167,9 @@ class FlexDimredNMAViewer(ProtocolViewer):
         return self._doViewRawDeformation(components)
 
     def _doViewRawDeformation(self, components):
+        ProtDeepHEMNMA = False
+        if (isinstance(self.protocol.inputNMA.get(), FlexProtDeepHEMNMAInfer)):
+            ProtDeepHEMNMA = True
         components = list(map(int, components.split()))
         dim = len(components)
         views = []
@@ -214,7 +206,12 @@ class FlexDimredNMAViewer(ProtocolViewer):
             else:
                 self.getData().YIND = modeList[1]
                 if dim == 2:
-                    plotter.plotArray2D("Normal-mode amplitudes in low-dimensional space: %s vs %s" % tuple(baseList),
+                    if ProtDeepHEMNMA:
+                        plotter.plotArray2D_xy(
+                            "Normal-mode amplitudes in low-dimensional space: %s vs %s" % tuple(baseList),
+                            *baseList)
+                    else:
+                        plotter.plotArray2D("Normal-mode amplitudes in low-dimensional space: %s vs %s" % tuple(baseList),
                                         *baseList)
                 elif dim == 3:
                     self.getData().ZIND = modeList[2]
@@ -225,6 +222,9 @@ class FlexDimredNMAViewer(ProtocolViewer):
         return views
 
     def _displayClustering(self, paramName):
+        ProtDeepHEMNMA = False
+        if (isinstance(self.protocol.inputNMA.get(), FlexProtDeepHEMNMAInfer)):
+            ProtDeepHEMNMA = True
         self.clusterWindow = self.tkWindow(ClusteringWindow,
                                            title='Clustering Tool',
                                            dim=self.protocol.reducedDim.get(),
@@ -240,10 +240,15 @@ class FlexDimredNMAViewer(ProtocolViewer):
                                            zlim_low=self.zlim_low,
                                            zlim_high=self.zlim_high,
                                            s=self.s,
-                                           alpha=self.alpha)
+                                           alpha=self.alpha,
+                                           deepHEMNMA=ProtDeepHEMNMA)
         return [self.clusterWindow]
 
     def _displayTrajectories(self, paramName):
+        ProtDeepHEMNMA = False
+        if (isinstance(self.protocol.inputNMA.get(), FlexProtDeepHEMNMAInfer)):
+            ProtDeepHEMNMA = True
+
         self.trajectoriesWindow = self.tkWindow(TrajectoriesWindow,
                                                 title='Trajectories Tool',
                                                 dim=self.protocol.reducedDim.get(),
@@ -261,7 +266,8 @@ class FlexDimredNMAViewer(ProtocolViewer):
                                                 zlim_low=self.zlim_low,
                                                 zlim_high=self.zlim_high,
                                                 s=self.s,
-                                                alpha=self.alpha)
+                                                alpha=self.alpha,
+                                                deepHEMNMA=ProtDeepHEMNMA)
         return [self.trajectoriesWindow]
 
     def _createCluster(self):
@@ -335,7 +341,13 @@ class FlexDimredNMAViewer(ProtocolViewer):
 
     def _generateAnimation(self):
         prot = self.protocol
-        projectorFile = prot.getProjectorFile()
+        # This is not getting the file correctly, we are workingaround it:
+        # projectorFile = prot.getProjectorFile()
+        projectorFile = prot._getExtraPath() + '/projector.txt'
+        if isfile(projectorFile):
+            print('Mapping found, the animation is exact inverse of the dimensionality reduction method')
+        else:
+            print('Mapping not found, the animation is an estimation of reversing the dimensionality reduction method')
 
         animation = self.trajectoriesWindow.getAnimationName()
         animationPath = prot._getExtraPath('animation_%s' % animation)
@@ -343,34 +355,53 @@ class FlexDimredNMAViewer(ProtocolViewer):
         cleanPath(animationPath)
         makePath(animationPath)
         animationRoot = join(animationPath, 'animation_%s' % animation)
-
         trajectoryPoints = np.array([p.getData() for p in self.trajectoriesWindow.pathData])
-        np.savetxt(join(animationPath, 'trajectory.txt'), trajectoryPoints)
 
-        if projectorFile:
+        if isfile(projectorFile):
             M = np.loadtxt(projectorFile)
-            deformations = np.dot(trajectoryPoints, np.linalg.pinv(M))
+            if prot.getMethodName() == 'sklearn_PCA':
+                pca = load(prot._getExtraPath('pca_pickled.txt'))
+                deformations = pca.inverse_transform(trajectoryPoints)
+            else:
+                deformations = np.dot(trajectoryPoints, np.linalg.pinv(M))
+                temp = np.loadtxt(prot._getExtraPath('deformations.txt')) # the original matrix file
+                deformations += np.outer(np.ones(deformations.shape[0]),np.mean(temp, axis=0))
+                temp = None
+            np.savetxt(animationRoot + 'trajectory.txt', trajectoryPoints)
         else:
             Y = np.loadtxt(prot.getOutputMatrixFile())
             X = np.loadtxt(prot.getDeformationFile())
             # Find closest points in deformations
             deformations = [X[np.argmin(np.sum((Y - p) ** 2, axis=1))] for p in trajectoryPoints]
 
-        pdb = prot.getInputPdb()
-        pdbFile = pdb.getFileName()
+        if prot.getDataChoice() == 'NMAs':
+            pdb = prot.getInputPdb()
+            pdbFile = pdb.getFileName()
+            modesFn = prot.getInputModes()
+            for i, d in enumerate(deformations):
+                atomsFn = animationRoot + 'atomsDeformed_%02d.pdb' % (i + 1)
+                cmd = '-o %s --pdb %s --nma %s --deformations ' % (atomsFn, pdbFile, modesFn)
+                for l in d:
+                    cmd += str(l) + ' '
+                # because it doesn't have an independent protocol we don't use self.runJob
+                runProgram('xmipp_pdb_nma_deform', cmd)
 
-        structureEM = prot.getInputPdb().getPseudoAtoms()
-        if not structureEM:
-            localFn = replaceBaseExt(basename(pdbFile), 'pdb')
-            cifToPdb(pdbFile, localFn)
-            pdbFile = basename(localFn)
-
-        modesFn = prot.inputNMA.get()._getExtraPath('modes.xmd')
-
-        for i, d in enumerate(deformations):
-            atomsFn = animationRoot + 'atomsDeformed_%02d.pdb' % (i + 1)
-            cmd = '-o %s --pdb %s --nma %s --deformations %s' % (atomsFn, pdbFile, modesFn, str(d)[1:-1])
-            runProgram('xmipp_pdb_nma_deform', cmd)
+        elif prot.getDataChoice() == 'PDBs':
+            # There is incompatibility issue with the rest of the code, we have to use the fahterPDB as one of the
+            # deformed PDBs (the first one)
+            # fatherPDB = prot._getExtraPath('pdb_file.pdb')
+            fatherPDB = prot._getExtraPath('generated_pdbs/000001.pdb')
+            lines_father = self.readPDB(fatherPDB)
+            list_father = self.PDB2List(lines_father)
+            i = 0
+            for line in deformations:
+                # reshaped pdb xyz coordinates
+                list_xyz = np.reshape(line, np.shape(list_father))
+                lines_i = self.list2PDBlines(list_xyz, lines_father)
+                atomsFn = animationRoot + 'atomsDeformed_%02d.pdb' % (i + 1)
+                self.writePDB(lines_i, atomsFn)
+                i += 1
+            pass
 
         # Join all deformations in a single pdb
         # iterating going up and down through all points
@@ -394,7 +425,7 @@ class FlexDimredNMAViewer(ProtocolViewer):
 
         trajFile.close()
         # Delete temporary atom files
-        cleanPattern(animationRoot + 'atomsDeformed_??.pdb')
+        # cleanPattern(animationRoot + 'atomsDeformed_??.pdb')
 
         # Generate the vmd script
         vmdFn = animationRoot + '.vmd'
@@ -420,9 +451,61 @@ class FlexDimredNMAViewer(ProtocolViewer):
         particles = self.protocol.getInputParticles()
 
         data = Data()
-        for i, particle in enumerate(particles):
-            data.addPoint(Point(pointId=particle.getObjId(),
-                                data=matrix[i, :],
-                                weight=particle._xmipp_cost.get()))
+
+        ProtDeepHEMNMA = False
+        if (isinstance(self.protocol.inputNMA.get(), FlexProtDeepHEMNMAInfer)):
+            ProtDeepHEMNMA = True
+
+        if ProtDeepHEMNMA:
+            for i, particle in enumerate(particles):
+                data.addPoint(Point(pointId=particle.getObjId(),
+                                    data=matrix[i, :],
+                                    weight=0.0))
+        else:
+            for i, particle in enumerate(particles):
+                data.addPoint(Point(pointId=particle.getObjId(),
+                                    data=matrix[i, :],
+                                    weight=particle._xmipp_cost.get()))
 
         return data
+
+    def readPDB(self, fnIn):
+        with open(fnIn) as f:
+            lines = f.readlines()
+        return lines
+
+    def PDB2List(self, lines):
+        newlines = []
+        for line in lines:
+            if line.startswith("ATOM "):
+                try:
+                    x = float(line[30:38])
+                    y = float(line[38:46])
+                    z = float(line[46:54])
+                    newline = [x, y, z]
+                    newlines.append(newline)
+                except:
+                    pass
+        return newlines
+
+    def list2PDBlines(self, list, lines):
+        newLines = []
+        i = 0
+        for line in lines:
+            if line.startswith("ATOM "):
+                try:
+                    x = list[i][0]
+                    y = list[i][1]
+                    z = list[i][2]
+                    newLine = line[0:30] + "%8.3f%8.3f%8.3f" % (x, y, z) + line[54:]
+                    i += 1
+                except:
+                    pass
+            else:
+                newLine = line
+            newLines.append(newLine)
+        return newLines
+
+    def writePDB(self, lines, fnOut):
+        with open(fnOut, mode='w') as f:
+            f.writelines(lines)
